@@ -2,6 +2,7 @@
 #define BOLOG_H
 
 #include <boost/math/special_functions/round.hpp>
+#include <boost/math/distributions/normal.hpp>
 
 namespace torali {
 
@@ -82,7 +83,60 @@ struct BoLog {
    gls[file_c * 3 + 1] = (float) gl[1];
    gls[file_c * 3] = (float) gl[2];
  }
- 
+
+
+  template<typename TConfig>
+  inline int32_t
+  _computeCNLs(TConfig const& c, double const mean, double const sd, float* gl, int32_t* gqval, int32_t const file_c) {
+    // Compute copy-number likelihoods
+    boost::math::normal s(mean, sd);
+    for(uint32_t geno=0; geno< MAX_CN; ++geno) {
+      double prob = boost::math::pdf(s, geno);
+      gl[file_c * MAX_CN + geno] = std::log10(prob);
+      gl[file_c * MAX_CN + geno] = (gl[file_c * MAX_CN + geno] > SMALLEST_GL) ? gl[file_c * MAX_CN + geno] : SMALLEST_GL;
+    }
+    uint32_t glBest=file_c * MAX_CN + 0;
+    uint32_t glBest2nd=file_c * MAX_CN + 1;
+    if (gl[glBest] < gl[glBest2nd]) {
+      glBest = file_c * MAX_CN + 1;
+      glBest2nd = file_c * MAX_CN + 0;
+    }
+    for(uint32_t geno=2; geno < MAX_CN; ++geno) {
+      if (gl[file_c * MAX_CN + geno] > gl[glBest2nd]) {
+	if (gl[file_c * MAX_CN + geno] > gl[glBest]) {
+	  glBest2nd = glBest;
+	  glBest = file_c * MAX_CN + geno;
+	} else {
+	  glBest2nd = file_c * MAX_CN + geno;
+	}
+      }
+    }
+    
+    // Variant quality
+    double glObs = std::log10(boost::math::pdf(s, mean));
+    glObs = (glObs > SMALLEST_GL) ? glObs : SMALLEST_GL;
+    uint32_t plVariant = (uint32_t) boost::math::round(-10 * glObs);
+    uint32_t plPloidy = (uint32_t) boost::math::round(-10 * gl[file_c * MAX_CN + c.ploidy]);
+    int32_t varqual = plPloidy - plVariant;
+    
+    // GQ
+    uint32_t plBest = (uint32_t) boost::math::round(-10 * gl[glBest]);
+    uint32_t plBest2nd = (uint32_t) boost::math::round(-10 * gl[glBest2nd]);
+    gqval[file_c] = plBest2nd - plBest;
+    
+    // Rescale by best genotype
+    double glBestVal = gl[glBest];
+    for(uint32_t geno=0; geno< MAX_CN; ++geno) gl[file_c * MAX_CN + geno] -= glBestVal;
+    
+    // Variant quality
+    return varqual;
+  }
+
+  template<typename TConfig>
+  inline int32_t
+  _computeCNLs(TConfig const& c, double const mean, double const sd, float* gl, int32_t* gqval) {
+    return _computeCNLs(c, mean, sd, gl, gqval, 0);
+  }
 
 }
 
