@@ -21,7 +21,6 @@
 #include <boost/icl/interval_map.hpp>
 #include <boost/filesystem.hpp>
 #include <boost/tokenizer.hpp>
-#include <boost/progress.hpp>
 #include <boost/accumulators/accumulators.hpp>
 #include <boost/accumulators/statistics.hpp>
 
@@ -73,7 +72,9 @@ classifyRun(TClassifyConfig const& c) {
   bcf_hdr_t* hdr = bcf_hdr_read(ifile);
 
   // Open output VCF file
-  htsFile *ofile = hts_open(c.outfile.string().c_str(), "wb");
+  std::string fmtout = "wb";
+  if (c.outfile.string() == "-") fmtout = "w";
+  htsFile *ofile = hts_open(c.outfile.string().c_str(), fmtout.c_str());
   bcf_hdr_t *hdr_out = bcf_hdr_dup(hdr);
   if (c.filter == "somatic") {
     bcf_hdr_remove(hdr_out, BCF_HL_INFO, "SOMATIC");
@@ -110,7 +111,7 @@ classifyRun(TClassifyConfig const& c) {
 
   // Parse BCF
   boost::posix_time::ptime now = boost::posix_time::second_clock::local_time();
-  std::cout << '[' << boost::posix_time::to_simple_string(now) << "] " << "Filtering VCF/BCF file" << std::endl;
+  std::cerr << '[' << boost::posix_time::to_simple_string(now) << "] " << "Filtering VCF/BCF file" << std::endl;
   bcf1_t* rec = bcf_init1();
   while (bcf_read(ifile, hdr, rec) == 0) {
     bcf_unpack(rec, BCF_UN_INFO);
@@ -144,7 +145,12 @@ classifyRun(TClassifyConfig const& c) {
     typedef std::vector<TCnSd> TSampleDist;
     TSampleDist control;
     TSampleDist tumor;
+    bool invalidCNV = false;
     for (int i = 0; i < bcf_hdr_nsamples(hdr); ++i) {
+      if ((!std::isfinite(rdcn[i])) || (rdcn[i] == -1)) {
+	invalidCNV = true;
+	break;
+      }
       if ((germline) || (c.controlSet.find(hdr->samples[i]) != c.controlSet.end())) {
 	// Control or population genomics
 	control.push_back(std::make_pair(rdcn[i], rdsd[i]));
@@ -153,6 +159,7 @@ classifyRun(TClassifyConfig const& c) {
 	tumor.push_back(std::make_pair(rdcn[i], rdsd[i]));
       }
     }
+    if (invalidCNV) continue;
 
     // Classify
     if (!germline) {
@@ -220,7 +227,7 @@ classifyRun(TClassifyConfig const& c) {
 	  for (int i = 0; i < bcf_hdr_nsamples(hdr); ++i) {
 	    rdcn[i] += cnshift;
 	    cnval[i] = boost::math::round(rdcn[i]);
-	    if (cnval[i] < MAX_CN) ++cncount[cnval[i]];
+	    if ((cnval[i] >= 0) && (cnval[i] < MAX_CN)) ++cncount[cnval[i]];
 	  }
 	}
 	
@@ -245,12 +252,18 @@ classifyRun(TClassifyConfig const& c) {
       // Re-compute CNLs
       std::vector<std::string> ftarr(bcf_hdr_nsamples(hdr));
       int32_t altqual = 0;
+      int32_t altcount = 0;
       for (int i = 0; i < bcf_hdr_nsamples(hdr); ++i) {
 	int32_t qval = _computeCNLs(c, rdcn[i], sd, cnl, gqval, i);
-	if (cnval[i] != c.ploidy) altqual += qval;
+	if (cnval[i] != c.ploidy) {
+	  altqual += qval;
+	  ++altcount;
+	}
 	if (gqval[i] < 15) ftarr[i] = "LowQual";
 	else ftarr[i] = "PASS";
       }
+      if (altcount == 0) continue;
+      altqual /= altcount;
       if (altqual < c.qual) continue;
       if (altqual > 10000) altqual = 10000;
       
@@ -287,7 +300,7 @@ classifyRun(TClassifyConfig const& c) {
   hts_close(ofile);
 
   // Build index
-  bcf_index_build(c.outfile.string().c_str(), 14);
+  if (c.outfile.string() != "-") bcf_index_build(c.outfile.string().c_str(), 14);
 
   // Close VCF
   bcf_hdr_destroy(hdr);
@@ -295,7 +308,7 @@ classifyRun(TClassifyConfig const& c) {
 
   // End
   now = boost::posix_time::second_clock::local_time();
-  std::cout << '[' << boost::posix_time::to_simple_string(now) << "] Done." << std::endl;
+  std::cerr << '[' << boost::posix_time::to_simple_string(now) << "] Done." << std::endl;
 
   return 0;
 }
@@ -309,7 +322,7 @@ int classify(int argc, char **argv) {
   generic.add_options()
     ("help,?", "show help message")
     ("filter,f", boost::program_options::value<std::string>(&c.filter)->default_value("somatic"), "Filter mode (somatic, germline)")
-    ("outfile,o", boost::program_options::value<boost::filesystem::path>(&c.outfile)->default_value("cnv.bcf"), "Filtered CNV BCF output file")
+    ("outfile,o", boost::program_options::value<boost::filesystem::path>(&c.outfile), "Filtered CNV BCF output file")
     ("minsize,m", boost::program_options::value<int32_t>(&c.minsize)->default_value(1000), "min. CNV size")
     ("maxsize,n", boost::program_options::value<int32_t>(&c.maxsize)->default_value(500000000), "max. CNV size")
     ("pass,p", "Filter sites for PASS")
@@ -351,9 +364,9 @@ int classify(int argc, char **argv) {
 
   // Check command line arguments
   if ((vm.count("help")) || (!vm.count("input-file"))) {
-    std::cout << std::endl;
-    std::cout << "Usage: delly " << argv[0] << " [OPTIONS] <input.bcf>" << std::endl;
-    std::cout << visible_options << "\n";
+    std::cerr << std::endl;
+    std::cerr << "Usage: delly " << argv[0] << " [OPTIONS] <input.bcf>" << std::endl;
+    std::cerr << visible_options << "\n";
     return 0;
   }
 
@@ -462,12 +475,20 @@ int classify(int argc, char **argv) {
     bcf_close(ifile);
   }
 
+  // Check outfile
+  if (!vm.count("outfile")) c.outfile = "-";
+  else {
+    if (c.outfile.string() != "-") {
+      if (!_outfileValid(c.outfile)) return 1;
+    }
+  }
+
   // Show cmd
   boost::posix_time::ptime now = boost::posix_time::second_clock::local_time();
-  std::cout << '[' << boost::posix_time::to_simple_string(now) << "] ";
-  std::cout << "delly ";
-  for(int i=0; i<argc; ++i) { std::cout << argv[i] << ' '; }
-  std::cout << std::endl;
+  std::cerr << '[' << boost::posix_time::to_simple_string(now) << "] ";
+  std::cerr << "delly ";
+  for(int i=0; i<argc; ++i) { std::cerr << argv[i] << ' '; }
+  std::cerr << std::endl;
 
   return classifyRun(c);
 }
