@@ -3,7 +3,6 @@
 
 #include <boost/filesystem.hpp>
 #include <boost/multi_array.hpp>
-#include <boost/progress.hpp>
 #include <boost/date_time/posix_time/posix_time.hpp>
 #include <boost/unordered_map.hpp>
 #include <boost/algorithm/string.hpp>
@@ -132,17 +131,20 @@ namespace torali
       double precndiff = std::abs((c.ploidy * precovsum / preexpcov) - (c.ploidy * succovsum / sucexpcov));
 
       // Intersect with delly SVs
-      int32_t searchWindow = std::max( std::max(1000, cnvs[n].start - cnvs[n].ciposlow), std::max(1000, cnvs[n].ciposhigh - cnvs[n].start));
       typename TSVs::const_iterator itbest = svbp[refIndex].end();
+      int32_t searchStart = std::max(0, std::min(cnvs[n-1].ciendlow, cnvs[n-1].end - 1000));
+      int32_t searchEnd = std::max(cnvs[n].ciposhigh, cnvs[n].start + 1000);
+      int32_t midpoint = (int32_t) ((cnvs[n-1].start + cnvs[n-1].end) / 2);
+      if (searchStart < midpoint) searchStart = midpoint;
+      midpoint = (int32_t) ((cnvs[n].start + cnvs[n].end) / 2);
+      if (searchEnd > midpoint) searchEnd = midpoint;
       // Current CNV start for this breakpoint
-      typename TSVs::const_iterator itsv = std::lower_bound(svbp[refIndex].begin(), svbp[refIndex].end(), SVBreakpoint(std::max(0, cnvs[n].start - searchWindow)), SortSVBreakpoint<SVBreakpoint>());
+      typename TSVs::const_iterator itsv = std::lower_bound(svbp[refIndex].begin(), svbp[refIndex].end(), SVBreakpoint(searchStart), SortSVBreakpoint<SVBreakpoint>());
       for(; itsv != svbp[refIndex].end(); ++itsv) {
-	if (itsv->pos - cnvs[n].start > searchWindow) break;
-	if (itsv->pos >= cnvs[n].end) continue;
-	if (itsv->pos <= cnvs[n-1].start) continue;
+	if (itsv->pos > searchEnd) break;
 	if ((itbest == svbp[refIndex].end()) || (itsv->qual > itbest->qual)) itbest = itsv;
       }
-      if ((itbest != svbp[refIndex].end()) && (itbest->qual > 50)) {
+      if ((itbest != svbp[refIndex].end()) && (itbest->qual >= 50)) {
 	// Check refined CNV
 	precovsum = 0;
 	preexpcov = 0;
@@ -570,7 +572,9 @@ namespace torali
     bam_hdr_t* bamhd = sam_hdr_read(samfile);
 
     // Output all copy-number variants
-    htsFile *fp = hts_open(c.cnvfile.string().c_str(), "wb");
+    std::string fmtout = "wb";
+    if (c.outfile.string() == "-") fmtout = "w";
+    htsFile *fp = hts_open(c.outfile.string().c_str(), fmtout.c_str());
     bcf_hdr_t *hdr = bcf_hdr_init("w");
 
     // Print vcf header
@@ -626,14 +630,11 @@ namespace torali
     
       // Iterate all structural variants
       now = boost::posix_time::second_clock::local_time();
-      std::cout << '[' << boost::posix_time::to_simple_string(now) << "] " << "Genotyping" << std::endl;
-      boost::progress_display show_progress( cnvs.size() );
+      std::cerr << '[' << boost::posix_time::to_simple_string(now) << "] " << "Genotyping" << std::endl;
       bcf1_t *rec = bcf_init();
       for(uint32_t i = 0; i < cnvs.size(); ++i) {
-	++show_progress;
-
 	// Invalid CNV?
-	if (cnvs[i].cn == -1) continue;
+	if ((!c.hasGenoFile) && (cnvs[i].cn == -1)) continue;
 
 	// Integer copy-number
 	int32_t absCN = (int32_t) boost::math::round(cnvs[i].cn);
@@ -721,7 +722,7 @@ namespace torali
     hts_close(fp);
     
     // Build index
-    bcf_index_build(c.cnvfile.string().c_str(), 14);
+    if (c.outfile.string() != "-") bcf_index_build(c.outfile.string().c_str(), 14);
   }
  
 
