@@ -9,7 +9,6 @@
 #include <boost/date_time/posix_time/posix_time.hpp>
 #include <boost/date_time/gregorian/gregorian.hpp>
 #include <boost/math/special_functions/round.hpp>
-#include <boost/progress.hpp>
 
 #include <htslib/sam.h>
 #include <htslib/faidx.h>
@@ -52,7 +51,7 @@ namespace torali
     std::string sampleName;
     boost::filesystem::path vcffile;
     boost::filesystem::path genofile;
-    boost::filesystem::path cnvfile;
+    boost::filesystem::path outfile;
     boost::filesystem::path covfile;
     boost::filesystem::path genome;
     boost::filesystem::path statsFile;
@@ -91,14 +90,15 @@ namespace torali
 
     // Parse BAM file
     boost::posix_time::ptime now = boost::posix_time::second_clock::local_time();
-    std::cout << '[' << boost::posix_time::to_simple_string(now) << "] " << "Count fragments" << std::endl;
-    boost::progress_display show_progress( hdr->n_targets );
+    std::cerr << '[' << boost::posix_time::to_simple_string(now) << "] " << "Count fragments" << std::endl;
 
     // Open output files
     boost::iostreams::filtering_ostream dataOut;
-    dataOut.push(boost::iostreams::gzip_compressor());
-    dataOut.push(boost::iostreams::file_sink(c.covfile.c_str(), std::ios_base::out | std::ios_base::binary));
-    dataOut << "chr\tstart\tend\t" << c.sampleName << "_mappable\t" << c.sampleName << "_counts\t" << c.sampleName << "_CN" << std::endl;
+    if (!c.covfile.empty()) {
+      dataOut.push(boost::iostreams::gzip_compressor());
+      dataOut.push(boost::iostreams::file_sink(c.covfile.c_str(), std::ios_base::out | std::ios_base::binary));
+      dataOut << "chr\tstart\tend\t" << c.sampleName << "_mappable\t" << c.sampleName << "_counts\t" << c.sampleName << "_CN" << std::endl;
+    }
 
     // CNVs
     std::vector<CNV> cnvs;
@@ -122,9 +122,8 @@ namespace torali
     faidx_t* faiMap = fai_load(c.mapFile.string().c_str());
     faidx_t* faiRef = fai_load(c.genome.string().c_str());
     for(int32_t refIndex=0; refIndex < (int32_t) hdr->n_targets; ++refIndex) {
-      ++show_progress;
-      if (chrNoData(c, refIndex, idx)) continue;
-
+      if ((!c.hasGenoFile) && (chrNoData(c, refIndex, idx))) continue;
+      
       // Check presence in mappability map
       std::string tname(hdr->target_name[refIndex]);
       int32_t seqlen = faidx_seq_len(faiMap, tname.c_str());
@@ -291,7 +290,7 @@ namespace torali
 		      double count = ((double) covsum / obsexp ) * (double) c.window_size / (double) winlen;
 		      double cn = c.ploidy;
 		      if (expcov > 0) cn = c.ploidy * covsum / expcov;
-		      dataOut << std::string(hdr->target_name[refIndex]) << "\t" << start << "\t" << (pos + 1) << "\t" << winlen << "\t" << count << "\t" << cn << std::endl;
+		      if (!c.covfile.empty()) dataOut << std::string(hdr->target_name[refIndex]) << "\t" << start << "\t" << (pos + 1) << "\t" << winlen << "\t" << count << "\t" << cn << std::endl;
 		      // reset
 		      covsum = 0;
 		      expcov = 0;
@@ -347,9 +346,9 @@ namespace torali
 		double count = ((double) covsum / obsexp ) * (double) (it->second - it->first) / (double) winlen;
 		double cn = c.ploidy;
 		if (expcov > 0) cn = c.ploidy * covsum / expcov;
-		dataOut << std::string(hdr->target_name[refIndex]) << "\t" << it->first << "\t" << it->second << "\t" << winlen << "\t" << count << "\t" << cn << std::endl;
+		if (!c.covfile.empty()) dataOut << std::string(hdr->target_name[refIndex]) << "\t" << it->first << "\t" << it->second << "\t" << winlen << "\t" << count << "\t" << cn << std::endl;
 	      } else {
-		dataOut << std::string(hdr->target_name[refIndex]) << "\t" << it->first << "\t" << it->second << "\tNA\tNA\tNA" << std::endl;
+		if (!c.covfile.empty()) dataOut << std::string(hdr->target_name[refIndex]) << "\t" << it->first << "\t" << it->second << "\tNA\tNA\tNA" << std::endl;
 	      }
 	    }
 	  }
@@ -374,7 +373,7 @@ namespace torali
 		double count = ((double) covsum / obsexp ) * (double) c.window_size / (double) winlen;
 		double cn = c.ploidy;
 		if (expcov > 0) cn = c.ploidy * covsum / expcov;
-		dataOut << std::string(hdr->target_name[refIndex]) << "\t" << start << "\t" << (pos + 1) << "\t" << winlen << "\t" << count << "\t" << cn << std::endl;
+		if (!c.covfile.empty()) dataOut << std::string(hdr->target_name[refIndex]) << "\t" << start << "\t" << (pos + 1) << "\t" << winlen << "\t" << count << "\t" << cn << std::endl;
 		// reset
 		covsum = 0;
 		expcov = 0;
@@ -422,7 +421,7 @@ namespace torali
 		double count = ((double) covsum / obsexp ) * (double) c.window_size / (double) winlen;
 		double cn = c.ploidy;
 		if (expcov > 0) cn = c.ploidy * covsum / expcov;
-		dataOut << std::string(hdr->target_name[refIndex]) << "\t" << start << "\t" << (start + c.window_size) << "\t" << winlen << "\t" << count << "\t" << cn << std::endl;
+		if (!c.covfile.empty()) dataOut << std::string(hdr->target_name[refIndex]) << "\t" << start << "\t" << (start + c.window_size) << "\t" << winlen << "\t" << count << "\t" << cn << std::endl;
 	      }
 	    }
 	  }
@@ -442,8 +441,10 @@ namespace torali
     bam_hdr_destroy(hdr);
     hts_idx_destroy(idx);
     sam_close(samfile);
-    dataOut.pop();
-    dataOut.pop();
+    if (!c.covfile.empty()) {
+      dataOut.pop();
+      dataOut.pop();
+    }
     
     return 0;
   }
@@ -460,8 +461,8 @@ namespace torali
       ("quality,q", boost::program_options::value<uint16_t>(&c.minQual)->default_value(10), "min. mapping quality")
       ("mappability,m", boost::program_options::value<boost::filesystem::path>(&c.mapFile), "input mappability map")
       ("ploidy,y", boost::program_options::value<uint16_t>(&c.ploidy)->default_value(2), "baseline ploidy")
-      ("outfile,o", boost::program_options::value<boost::filesystem::path>(&c.cnvfile)->default_value("cnv.bcf"), "output CNV file")
-      ("covfile,c", boost::program_options::value<boost::filesystem::path>(&c.covfile)->default_value("cov.gz"), "output coverage file")
+      ("outfile,o", boost::program_options::value<boost::filesystem::path>(&c.outfile), "BCF output file")
+      ("covfile,c", boost::program_options::value<boost::filesystem::path>(&c.covfile), "gzipped coverage file")
       ;
 
     boost::program_options::options_description cnv("CNV calling");
@@ -516,18 +517,18 @@ namespace torali
 
     // Check command line arguments
     if ((vm.count("help")) || (!vm.count("input-file")) || (!vm.count("genome")) || (!vm.count("mappability"))) {
-      std::cout << std::endl;
-      std::cout << "Usage: delly " << argv[0] << " [OPTIONS] -g <genome.fa> -m <genome.map> <aligned.bam>" << std::endl;
-      std::cout << visible_options << "\n";
+      std::cerr << std::endl;
+      std::cerr << "Usage: delly " << argv[0] << " [OPTIONS] -g <genome.fa> -m <genome.map> <aligned.bam>" << std::endl;
+      std::cerr << visible_options << "\n";
       return 1;
     }
 
     // Show cmd
     boost::posix_time::ptime now = boost::posix_time::second_clock::local_time();
-    std::cout << '[' << boost::posix_time::to_simple_string(now) << "] ";
-    std::cout << "delly ";
-    for(int i=0; i<argc; ++i) { std::cout << argv[i] << ' '; }
-    std::cout << std::endl;
+    std::cerr << '[' << boost::posix_time::to_simple_string(now) << "] ";
+    std::cerr << "delly ";
+    for(int i=0; i<argc; ++i) { std::cerr << argv[i] << ' '; }
+    std::cerr << std::endl;
 
     // Stats file
     if (vm.count("statsfile")) c.hasStatsFile = true;
@@ -599,6 +600,14 @@ namespace torali
       bcf_close(ifile);
       c.hasVcfFile = true;
     } else c.hasVcfFile = false;
+
+    // Check outfile
+    if (!vm.count("outfile")) c.outfile = "-";
+    else {
+      if (c.outfile.string() != "-") {
+	if (!_outfileValid(c.outfile)) return 1;
+      }
+    }
     
     // Check bam file
     LibraryInfo li;
@@ -705,6 +714,26 @@ namespace torali
       typedef std::vector<TWindowCounts> TGenomicWindowCounts;
       TGenomicWindowCounts scanCounts(c.nchr, TWindowCounts());
       scan(c, li, scanCounts);
+
+      // Check coverage
+      {
+	std::vector<uint32_t> sampleScanVec;
+	for(uint32_t i = 0; i < scanCounts.size(); ++i) {
+	  for(uint32_t j = 0; j < scanCounts[i].size(); ++j) {
+	    sampleScanVec.push_back(scanCounts[i][j].cov);
+	  }
+	  if (sampleScanVec.size() > 1000000) break;
+	}
+	std::sort(sampleScanVec.begin(), sampleScanVec.end());
+	if (sampleScanVec.empty()) {
+	  std::cerr << "Not enough windows!" << std::endl;
+	  return 1;
+	}
+	if (sampleScanVec[sampleScanVec.size()/2] < 5) {
+	  std::cerr << "Please increase the window size. Coverage is too low!" << std::endl;
+	  return 1;
+	}
+      }
     
       // Select stable windows
       selectWindows(c, scanCounts);
@@ -751,7 +780,7 @@ namespace torali
 
     // Done
     now = boost::posix_time::second_clock::local_time();
-    std::cout << '[' << boost::posix_time::to_simple_string(now) << "] " << "Done." << std::endl;
+    std::cerr << '[' << boost::posix_time::to_simple_string(now) << "] " << "Done." << std::endl;
 
     return 0;
   }
