@@ -23,6 +23,10 @@
 #include "coral.h"
 #include "asmode.h"
 #include "dpe.h"
+#include "pangenome.h"
+#include "markdup.h"
+#include "compvcf.h"
+#include "chimera.h"
 
 using namespace torali;
 
@@ -39,14 +43,22 @@ displayUsage() {
   std::cerr << "Long-read SV calling:" << std::endl;
   std::cerr << "    lr           long-read SV discovery" << std::endl;
   std::cerr << std::endl;
-  //std::cerr << "Assembly-based SV calling:" << std::endl;
-  //std::cerr << "    asm          assembly SV discovery" << std::endl;
+  //std::cerr << "Pan-genome based SV calling (work-in-progress):" << std::endl;
+  //std::cerr << "    pg           pan-genome SV discovery" << std::endl;
+  //std::cerr << std::endl;
+  //std::cerr << "Assembly-based SV calling (work-in-progress):" << std::endl;
+  //std::cerr << "    asm          assembly SV site discovery" << std::endl;
   //std::cerr << std::endl;
   std::cerr << "Copy-number variant calling:" << std::endl;
   std::cerr << "    cnv          discover and genotype copy-number variants" << std::endl;
   std::cerr << "    classify     classify somatic or germline copy-number variants" << std::endl;
+  //std::cerr << std::endl;
+  //std::cerr << "Multi-sample VCF operations:" << std::endl;
+  //std::cerr << "    markdup      mark duplicate SV sites based on SV allele and GT concordance" << std::endl;
+  //std::cerr << "    compvcf      compare multi-sample VCF file to a ground truth VCF file" << std::endl;
   //std::cerr << "Deprecated:" << std::endl;
   //std::cerr << "    dpe          double paired-end signatures" << std::endl;
+  //std::cerr << "    chimera      ONT chimera flagging" << std::endl;
   //std::cerr << std::endl;
   std::cerr << std::endl;
   std::cerr << std::endl;
@@ -87,9 +99,21 @@ int main(int argc, char **argv) {
     else if ((std::string(argv[1]) == "asm")) {
       return asmode(argc-1,argv+1);
     }
+    else if ((std::string(argv[1]) == "markdup")) {
+      return markdup(argc-1,argv+1);
+    }
+    else if ((std::string(argv[1]) == "compvcf")) {
+      return compvcf(argc-1,argv+1);
+    }
+    else if ((std::string(argv[1]) == "pg")) {
+      return pg(argc-1,argv+1);
+    }
     else if ((std::string(argv[1]) == "dpe")) {
       return dpe(argc-1,argv+1);
     }
+    else if ((std::string(argv[1]) == "chimera")) {
+      return chimera(argc-1,argv+1);
+    }
     else if ((std::string(argv[1]) == "cnv")) {
       return coral(argc-1,argv+1);
     }
