@@ -240,7 +240,7 @@ namespace torali
 		    else isizelen = (it->second[j].seqpos - it->second[i].seqpos) - (it->second[i].refpos - it->second[j].refpos);
 		  } else isizelen = 0;
 		}
-		if ((isizelen > (int32_t) c.minRefSep) && (isizelen < (int32_t) c.maxInsertionSize)) {
+		if ((isizelen > (int32_t) c.minRefSep) && (isizelen < std::max(it->second[i].seqpos, it->second[j].seqpos))) {
 		  // Avg. qval
 		  int32_t qval = (int32_t) (((int32_t) it->second[i].qual + (int32_t) it->second[j].qual) / 2);
 		  if (it->second[i].refpos <= it->second[j].refpos) {
@@ -310,6 +310,41 @@ namespace torali
   }
 
 
+  // Insertion junctions
+  template<typename TConfig, typename TReadBp>
+  inline void
+  bridgeInsertions(TConfig const& c, TReadBp const& readBp, std::vector<std::vector<SRBamRecord> >& br) {
+    // Insertion map
+    std::set<std::size_t> readIds;
+    typedef std::map<uint32_t, int32_t> TPosInsMap;
+    TPosInsMap pins;
+    for(int32_t refIndex = 0; refIndex < c.nchr; ++refIndex) {
+      pins.clear();
+      readIds.clear();
+      for(uint32_t i = 0; i < br[4].size(); ++i) {
+	if (br[4][i].chr == refIndex) {
+	  readIds.insert(br[4][i].id);
+	  for(int32_t k = br[4][i].pos; k <= br[4][i].pos2; ++k) {
+	    typename TPosInsMap::iterator it = pins.find(k);
+	    if (it == pins.end()) pins.insert(std::make_pair(k, br[4][i].inslen));
+	    else it->second = (it->second + br[4][i].inslen) / 2;
+	  }
+	}
+      }
+      for(typename TReadBp::const_iterator it = readBp.begin(); it != readBp.end(); ++it) {
+	int32_t rst = _selectReadStart(it->second);
+	for(uint32_t i = 0; i < it->second.size(); ++i) {
+	  // Any insertion?
+	  if ((it->second[i].refidx == refIndex) && (readIds.find(it->first) == readIds.end()) && (pins.find(it->second[i].refpos) != pins.end())) {
+	    int32_t qval = (int32_t) it->second[i].qual;
+	    br[4].push_back(SRBamRecord(it->second[i].refidx, it->second[i].refpos, it->second[i].refidx, it->second[i].refpos + 1, rst, it->second[i].seqpos, qval, pins[it->second[i].refpos], it->first));
+	  }
+	}
+      }
+    }
+  }
+  
+
   template<typename TConfig, typename TValidRegion, typename TReadBp>
   inline void
   findJunctions(TConfig const& c, TValidRegion const& validRegions, TReadBp& readBp) {
@@ -462,7 +497,10 @@ namespace torali
     if ((c.svtset.empty()) || (c.svtset.find(2) != c.svtset.end())) selectDeletions(c, readBp, br);
     if ((c.svtset.empty()) || (c.svtset.find(3) != c.svtset.end())) selectDuplications(c, readBp, br);
     if ((c.svtset.empty()) || (c.svtset.find(0) != c.svtset.end()) || (c.svtset.find(1) != c.svtset.end())) selectInversions(c, readBp, br);
-    if ((c.svtset.empty()) || (c.svtset.find(4) != c.svtset.end())) selectInsertions(c, readBp, br);
+    if ((c.svtset.empty()) || (c.svtset.find(4) != c.svtset.end())) {
+      selectInsertions(c, readBp, br);
+      bridgeInsertions(c, readBp, br);
+    }
     if ((c.svtset.empty()) || (c.svtset.find(5) != c.svtset.end()) || (c.svtset.find(6) != c.svtset.end()) || (c.svtset.find(7) != c.svtset.end()) || (c.svtset.find(8) != c.svtset.end())) selectTranslocations(c, readBp, br);
   }
 
@@ -488,7 +526,7 @@ namespace torali
     _findSRBreakpoints(c, validRegions, srBR);
 	 	 
     // Debug
-    //outputSRBamRecords(c, srBR);
+    //outputSRBamRecords(c, srBR, true);
 
     // Cluster BAM records
     for(uint32_t svt = 0; svt < srBR.size(); ++svt) {
@@ -498,10 +536,10 @@ namespace torali
       std::sort(srBR[svt].begin(), srBR[svt].end(), SortSRBamRecord<SRBamRecord>());
       
       // Cluster
-      cluster(c, srBR[svt], svc, c.maxReadSep, svt);
+      cluster(c, srBR[svt], svc, svt);
 
       // Debug
-      //outputStructuralVariants(c, svc, srBR, svt);
+      //outputStructuralVariants(c, svc, srBR, svt, true);
       // Track split-reads
       samFile* samfile = sam_open(c.files[0].string().c_str(), "r");
       bam_hdr_t* hdr = sam_hdr_read(samfile);
@@ -529,30 +567,65 @@ namespace torali
 
   template<typename TConfig>
   inline void
-  outputSRBamRecords(TConfig const& c, std::vector<std::vector<SRBamRecord> > const& br) {
-    samFile* samfile = sam_open(c.files[0].string().c_str(), "r");
-    hts_set_fai_filename(samfile, c.genome.string().c_str());
-    bam_hdr_t* hdr = sam_hdr_read(samfile);
-
+  outputSRBamRecords(TConfig const& c, std::vector<std::vector<SRBamRecord> > const& br, bool const longread) {
     // Header
-    std::cerr << "id\tchr1\tpos1\tchr2\tpos2\tsvtype\tct\tinslen" << std::endl;
+    std::cerr << "qname\tid\tchr1\tpos1\tchr2\tpos2\tsvtype\tct\tqual\tinslen" << std::endl;
 
+    // Hash reads
+    typedef std::map<std::size_t, std::string> THashMap;
+    THashMap hm;
+    typedef std::vector<samFile*> TSamFile;
+    typedef std::vector<hts_idx_t*> TIndex;
+    TSamFile samfile(c.files.size());
+    TIndex idx(c.files.size());
+    for(unsigned int file_c = 0; file_c < c.files.size(); ++file_c) {
+      samfile[file_c] = sam_open(c.files[file_c].string().c_str(), "r");
+      hts_set_fai_filename(samfile[file_c], c.genome.string().c_str());
+      idx[file_c] = sam_index_load(samfile[file_c], c.files[file_c].string().c_str());
+    }
+    bam_hdr_t* hdr = sam_hdr_read(samfile[0]);
+    for(int32_t refIndex = 0; refIndex < hdr->n_targets; ++refIndex) {
+      for(unsigned int file_c = 0; file_c < c.files.size(); ++file_c) {
+	hts_itr_t* iter = sam_itr_queryi(idx[file_c], refIndex, 0, hdr->target_len[refIndex]);
+	bam1_t* rec = bam_init1();
+	while (sam_itr_next(samfile[file_c], iter, rec) >= 0) {
+	  if (rec->core.flag & (BAM_FQCFAIL | BAM_FDUP | BAM_FUNMAP)) continue;
+	  std::size_t seed;
+	  if (longread) seed = hash_lr(rec);
+	  else seed = hash_sr(rec);
+	  std::string qname = bam_get_qname(rec);
+	  if (hm.find(seed) == hm.end()) hm.insert(std::make_pair(seed, qname));
+	  else {
+	    if (hm[seed] != qname) {
+	      std::cerr << "Warning: Hash collision! " << seed << ',' << hm[seed] << ',' << qname << std::endl;
+	    }
+	  }
+	}
+	bam_destroy1(rec);
+	hts_itr_destroy(iter);
+      }
+    }
+    
     // SVs
     for(uint32_t svt = 0; svt < br.size(); ++svt) {
       for(uint32_t i = 0; i < br[svt].size(); ++i) {
-	std::cerr << br[svt][i].id << '\t' << hdr->target_name[br[svt][i].chr] << '\t' << br[svt][i].pos << '\t' << hdr->target_name[br[svt][i].chr2] << '\t' << br[svt][i].pos2 << '\t' << _addID(svt) << '\t' << _addOrientation(svt) << '\t' << br[svt][i].inslen << std::endl;
+	std::cerr << hm[br[svt][i].id] << '\t' << br[svt][i].id << '\t' << hdr->target_name[br[svt][i].chr] << '\t' << br[svt][i].pos << '\t' << hdr->target_name[br[svt][i].chr2] << '\t' << br[svt][i].pos2 << '\t' << _addID(svt) << '\t' << _addOrientation(svt) << '\t' << br[svt][i].qual << '\t' << br[svt][i].inslen << std::endl;
       }
     }
+
     // Clean-up
     bam_hdr_destroy(hdr);
-    sam_close(samfile);
+    for(unsigned int file_c = 0; file_c < c.files.size(); ++file_c) {
+      hts_idx_destroy(idx[file_c]);
+      sam_close(samfile[file_c]);
+    }
   }
 
   template<typename TConfig, typename TSvtSRBamRecord>
   inline void
-  outputStructuralVariants(TConfig const& c, std::vector<StructuralVariantRecord> const& svs, TSvtSRBamRecord const& srBR, int32_t const svt) {
+  outputStructuralVariants(TConfig const& c, std::vector<StructuralVariantRecord> const& svs, TSvtSRBamRecord const& srBR, int32_t const svt, bool const longread) {
     // Header
-    std::cerr << "chr1\tpos1\tchr2\tpos2\tsvtype\tct\tpeSupport\tsrSupport" << std::endl;
+    std::cerr << "chr1\tpos1\tchr2\tpos2\tsvtype\tct\tinslen\tpeSupport\tsrSupport" << std::endl;
     
     // Hash reads
     typedef std::map<std::size_t, std::string> THashMap;
@@ -573,7 +646,9 @@ namespace torali
 	bam1_t* rec = bam_init1();
 	while (sam_itr_next(samfile[file_c], iter, rec) >= 0) {
 	  if (rec->core.flag & (BAM_FQCFAIL | BAM_FDUP | BAM_FUNMAP)) continue;
-	  std::size_t seed = hash_lr(rec);
+	  std::size_t seed;
+	  if (longread) seed = hash_lr(rec);
+	  else seed = hash_sr(rec);
 	  std::string qname = bam_get_qname(rec);
 	  if (hm.find(seed) == hm.end()) hm.insert(std::make_pair(seed, qname));
 	  else {
@@ -600,7 +675,7 @@ namespace torali
     // SVs
     for(uint32_t i = 0; i < svs.size(); ++i) {
       if (svs[i].svt != svt) continue;
-      std::cerr << hdr->target_name[svs[i].chr] << '\t' << svs[i].svStart << '\t' << hdr->target_name[svs[i].chr2] << '\t' << svs[i].svEnd << '\t' << _addID(svs[i].svt) << '\t' << _addOrientation(svs[i].svt) << '\t' << svs[i].peSupport << '\t' << svs[i].srSupport << '\t';
+      std::cerr << hdr->target_name[svs[i].chr] << '\t' << svs[i].svStart << '\t' << hdr->target_name[svs[i].chr2] << '\t' << svs[i].svEnd << '\t' << _addID(svs[i].svt) << '\t' << _addOrientation(svs[i].svt) << '\t' << svs[i].insLen << '\t' << svs[i].peSupport << '\t' << svs[i].srSupport << '\t';
       for(uint32_t k = 0; k < svReadNames[svs[i].id].size(); ++k) std::cerr << svReadNames[svs[i].id][k] << ',';
       std::cerr << std::endl;
     }
@@ -614,34 +689,6 @@ namespace torali
   }
 
 
-  template<typename TConfig>
-  inline void
-  outputStructuralVariants(TConfig const& c, std::vector<StructuralVariantRecord> const& svs, int32_t const svt) {
-    samFile* samfile = sam_open(c.files[0].string().c_str(), "r");
-    hts_set_fai_filename(samfile, c.genome.string().c_str());
-    bam_hdr_t* hdr = sam_hdr_read(samfile);
-
-    // Header
-    std::cerr << "chr1\tpos1\tchr2\tpos2\tsvtype\tct\tpeSupport\tsrSupport\tconsensus" << std::endl;
-    
-    // SVs
-    for(uint32_t i = 0; i < svs.size(); ++i) {
-      if (svs[i].svt != svt) continue;
-      std::cerr << hdr->target_name[svs[i].chr] << '\t' << svs[i].svStart << '\t' << hdr->target_name[svs[i].chr2] << '\t' << svs[i].svEnd << '\t' << _addID(svs[i].svt) << '\t' << _addOrientation(svs[i].svt) << '\t' << svs[i].peSupport << '\t' << svs[i].srSupport << '\t' << svs[i].consensus << std::endl;
-    }
-    
-    // Clean-up
-    bam_hdr_destroy(hdr);
-    sam_close(samfile);
-  }
-
-  template<typename TConfig>
-  inline void
-  outputStructuralVariants(TConfig const& c, std::vector<StructuralVariantRecord> const& svs) {
-    for(uint32_t i = 0; i <= 8; ++i) outputStructuralVariants(c, svs, i);
-  }
-  
-
 }
 
 #endif
