@@ -49,6 +49,7 @@ struct MergeConfig {
   uint32_t minsize;
   uint32_t maxsize;
   uint32_t coverage;
+  int32_t qualthres;
   float recoverlap;
   float vaf;
   boost::filesystem::path outfile;
@@ -145,6 +146,9 @@ void _fillIntervalMap(MergeConfig const& c, TGenomeIntervals& iScore, TContigMap
       if (bcf_get_info_flag(hdr, rec, "PRECISE", 0, 0) > 0) precise=true;
       if ((c.filterForPrecise) && (!precise)) continue;
 
+      // Quality threshold
+      if (rec->qual < c.qualthres) continue;
+      
       // Variant allele frequency filter
       if ((c.vaf > 0) || (c.coverage > 0)) {
 	float maxvaf = 0;
@@ -279,6 +283,7 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
   bcf_hdr_append(hdr_out, "##INFO=<ID=SR,Number=1,Type=Integer,Description=\"Split-read support\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=SRQ,Number=1,Type=Float,Description=\"Split-read consensus alignment quality\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=CONSENSUS,Number=1,Type=String,Description=\"Split-read consensus sequence\">");
+  bcf_hdr_append(hdr_out, "##INFO=<ID=CONSBP,Number=1,Type=Integer,Description=\"Consensus SV breakpoint position\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=CE,Number=1,Type=Float,Description=\"Consensus sequence entropy\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=CT,Number=1,Type=String,Description=\"Paired-end signature induced connection type\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=SVLEN,Number=1,Type=Integer,Description=\"Insertion length for SVTYPE=INS.\">");
@@ -343,6 +348,8 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
   int32_t* inslen = NULL;
   int32_t npos2 = 0;
   int32_t* pos2 = NULL;
+  int32_t nconsbp = 0;
+  int32_t* consbp = NULL;
   int32_t nhomlen = 0;
   int32_t* homlen = NULL;
   int32_t nmapq = 0;
@@ -389,16 +396,20 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
       if ((c.filterForPrecise) && (!precise)) passPrecise = false;
 
       // Check PASS and precise
-      if ((passPrecise) && (pass)) {
+      if ((rec[idx]->qual >= c.qualthres) && (passPrecise) && (pass)) {
 	// Correct size
 	std::string chrName(bcf_hdr_id2name(hdr[idx], rec[idx]->rid));
 	uint32_t tid = cMap[chrName];
 	uint32_t svStart = rec[idx]->pos;
 	uint32_t svEnd = svStart + 1;
+	uint32_t tmpSvEnd = 0;
 	if (bcf_get_info_int32(hdr[idx], rec[idx], "END", &svend, &nsvend) > 0) svEnd = *svend;
 	unsigned int inslenVal = 0;
 	if (bcf_get_info_int32(hdr[idx], rec[idx], "INSLEN", &inslen, &ninslen) > 0) inslenVal = *inslen;
-	if (recsvt == 4) svEnd = svStart + inslenVal; // To enable reciprocal overlap
+	if (recsvt == 4) {
+	  tmpSvEnd = svEnd;
+	  svEnd = svStart + inslenVal; // To enable reciprocal overlap
+	}
 
 	// Parse INFO fields
 	if ((std::string(svt) == "BND") || ((std::string(svt) == "INS") && (inslenVal >= c.minsize) && (inslenVal <= c.maxsize)) || ((std::string(svt) != "BND") && (std::string(svt) != "INS") && (svEnd - svStart >= c.minsize) && (svEnd - svStart <= c.maxsize))) {
@@ -446,9 +457,11 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
 
 	    std::string consensus;
 	    float ceVal = 0;
+	    int32_t consBpVal = 0;
 	    if (precise) {
 	      if (bcf_get_info_float(hdr[idx], rec[idx], "CE", &ce, &nce) > 0) ceVal = *ce;
 	      if (bcf_get_info_string(hdr[idx], rec[idx], "CONSENSUS", &cons, &ncons) > 0) consensus = boost::to_upper_copy(std::string(cons));
+	      if (bcf_get_info_int32(hdr[idx], rec[idx], "CONSBP", &consbp, &nconsbp) > 0) consBpVal = *consbp;
 	    }
 	    
 	    // Create new record
@@ -478,7 +491,8 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
 	    std::string dellyVersion("EMBL.DELLYv");
 	    dellyVersion += dellyVersionNumber;
 	    bcf_update_info_string(hdr_out,rout, "SVMETHOD", dellyVersion.c_str());
-	    bcf_update_info_int32(hdr_out, rout, "END", &svEnd, 1);
+	    if (recsvt == 4) bcf_update_info_int32(hdr_out, rout, "END", &tmpSvEnd, 1);
+	    else bcf_update_info_int32(hdr_out, rout, "END", &svEnd, 1);
 	    if (svtin >= DELLY_SVT_TRANS) {
 	      bcf_update_info_string(hdr_out,rout, "CHR2", chr2Name.c_str());
 	      bcf_update_info_int32(hdr_out, rout, "POS2", &pos2val, 1);
@@ -502,6 +516,7 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
 	      if (consensus.size()) {
 		bcf_update_info_string(hdr_out, rout, "CONSENSUS", consensus.c_str());
 		bcf_update_info_float(hdr_out, rout, "CE", &ceVal, 1);
+		bcf_update_info_int32(hdr_out, rout, "CONSBP", &consBpVal, 1);
 	      }
 	    }
 	
@@ -527,6 +542,7 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
   if (homlen != NULL) free(homlen);
   if (inslen != NULL) free(inslen);
   if (pos2 != NULL) free(pos2);
+  if (consbp != NULL) free(consbp);
   if (mapq != NULL) free(mapq);
   if (srmapq != NULL) free(srmapq);
   if (ct != NULL) free(ct);
@@ -664,7 +680,7 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
 	if ((c.filterForPrecise) && (!precise)) passPrecise = false;
 
 	// Check PASS and precise
-	if ((passPrecise) && (pass)) {
+	if ((rec[idx]->qual >= c.qualthres) && (passPrecise) && (pass)) {
 	  // Correct size
 	  std::string chrName(bcf_hdr_id2name(hdr[idx], rec[idx]->rid));
 	  uint32_t tid = cMap[chrName];
@@ -897,9 +913,10 @@ int merge(int argc, char **argv) {
   generic.add_options()
     ("help,?", "show help message")
     ("outfile,o", boost::program_options::value<boost::filesystem::path>(&c.outfile), "Merged SV BCF output file")
+    ("quality,y", boost::program_options::value<int32_t>(&c.qualthres)->default_value(200), "min. SV site quality")
     ("chunks,u", boost::program_options::value<uint32_t>(&c.chunksize)->default_value(500), "max. chunk size to merge groups of BCF files")
     ("vaf,a", boost::program_options::value<float>(&c.vaf)->default_value(0.15), "min. fractional ALT support")
-    ("coverage,v", boost::program_options::value<uint32_t>(&c.coverage)->default_value(10), "min. coverage")
+    ("coverage,v", boost::program_options::value<uint32_t>(&c.coverage)->default_value(5), "min. coverage")
     ("minsize,m", boost::program_options::value<uint32_t>(&c.minsize)->default_value(0), "min. SV size")
     ("maxsize,n", boost::program_options::value<uint32_t>(&c.maxsize)->default_value(1000000), "max. SV size")
     ("cnvmode,e", "Merge delly CNV files")
