@@ -155,16 +155,13 @@ vcfParse(TConfig const& c, bam_hdr_t* hd, std::vector<TStructuralVariantRecord>&
   bcf_hdr_t* hdr = bcf_hdr_read(ifile);
   bcf1_t* rec = bcf_init();
 
-  // Parse genome if necessary
-  faidx_t* fai = fai_load(c.genome.string().c_str());
-  char* seq = NULL;
-  int32_t lastRefIndex = -1;
-  
   // Parse bcf
   int32_t nsvend = 0;
   int32_t* svend = NULL;
   int32_t npos2 = 0;
   int32_t* pos2 = NULL;
+  int32_t nconsbp = 0;
+  int32_t* consbp = NULL;
   int32_t nsvlen = 0;
   int32_t* svlen = NULL;
   int32_t npe = 0;
@@ -191,21 +188,22 @@ vcfParse(TConfig const& c, bam_hdr_t* hd, std::vector<TStructuralVariantRecord>&
   char* cons = NULL;
   int32_t nchr2 = 0;
   char* chr2 = NULL;
-  uint16_t wimethod = 0; 
+  bool dellyVCF = false;
   while (bcf_read(ifile, hdr, rec) == 0) {
     bcf_unpack(rec, BCF_UN_INFO);
 
     // Delly BCF file?
-    if (!wimethod) {
-      wimethod = 2;
+    if (!dellyVCF) {
       if (bcf_get_info_string(hdr, rec, "SVMETHOD", &method, &nmethod) > 0) {
 	std::string mstr = std::string(method);
-	if ((mstr.size() >= 10) && (mstr.substr(0, 10)  == "EMBL.DELLY")) wimethod = 1;
+	if ((mstr.size() >= 10) && (mstr.substr(0, 10)  == "EMBL.DELLY")) {
+	  if (_isKeyPresent(hdr, "CONSBP")) dellyVCF = true;
+	}
       }
     }
 
     // Delly
-    if (wimethod == 1) {
+    if (dellyVCF) {
       // Fill SV record
       StructuralVariantRecord svRec;
       std::string chrName = bcf_hdr_id2name(hdr, rec->rid);
@@ -215,8 +213,12 @@ vcfParse(TConfig const& c, bam_hdr_t* hd, std::vector<TStructuralVariantRecord>&
       svRec.id = svs.size();
       svRec.mapq = rec->qual;
       std::string refAllele = rec->d.allele[0];
-      std::string altAllele = rec->d.allele[1];
-      svRec.alleles = refAllele + "," + altAllele;
+      if ((!refAllele.empty()) && (refAllele != ".")) {
+	std::string altAllele = rec->d.allele[1];
+	if ((!altAllele.empty()) && (altAllele != ".")) {
+	  svRec.alleles = refAllele + "," + altAllele;
+	}
+      }
 
       // Parse SV type
       if ((bcf_get_info_string(hdr, rec, "SVTYPE", &svt, &nsvt) > 0) && (bcf_get_info_string(hdr, rec, "CT", &ct, &nct) > 0)) svRec.svt = _decodeOrientation(std::string(ct), std::string(svt));
@@ -249,6 +251,7 @@ vcfParse(TConfig const& c, bam_hdr_t* hd, std::vector<TStructuralVariantRecord>&
       if (svRec.svt < DELLY_SVT_TRANS) {
 	// Intra-chromosomal SV
 	if (bcf_get_info_int32(hdr, rec, "END", &svend, &nsvend) > 0) svRec.svEnd = *svend;
+	svRec.svEnd += 1;
       } else {
 	// Inter-chromosomal SV
 	if (bcf_get_info_string(hdr, rec, "CHR2", &chr2, &nchr2) > 0) {
@@ -257,7 +260,10 @@ vcfParse(TConfig const& c, bam_hdr_t* hd, std::vector<TStructuralVariantRecord>&
 	}
 	if (bcf_get_info_int32(hdr, rec, "POS2", &pos2, &npos2) > 0) svRec.svEnd = *pos2;
       }
-      if (bcf_get_info_string(hdr, rec, "CONSENSUS", &cons, &ncons) > 0) svRec.consensus = std::string(cons);
+      if (bcf_get_info_string(hdr, rec, "CONSENSUS", &cons, &ncons) > 0) {
+	svRec.consensus = std::string(cons);
+	if (bcf_get_info_int32(hdr, rec, "CONSBP", &consbp, &nconsbp) > 0) svRec.consBp = *consbp;
+      }
       else svRec.precise = false;
       if (bcf_get_info_int32(hdr, rec, "CIPOS", &cipos, &ncipos) > 0) {
 	svRec.ciposlow = cipos[0];
@@ -281,96 +287,15 @@ vcfParse(TConfig const& c, bam_hdr_t* hd, std::vector<TStructuralVariantRecord>&
       else svRec.srAlignQuality = 0;
 
       svs.push_back(svRec);
-    } else if (wimethod == 2) {
-      // Assume precise SV, only deletions supported and INFO:END is required!!!
-      if (rec->n_allele == 2) {
-	std::string refAllele = rec->d.allele[0];
-	std::string altAllele = rec->d.allele[1];
-	StructuralVariantRecord svRec;
-	bool tagUse;
-	bool insertion = false;
-	if (altAllele == "<DEL>") {
-	  svRec.svt = 2;
-	  tagUse = true;
-	} else if (altAllele == "<INS>") {
-	  // No precise insertion sequence, cannot be genotyped by Delly
-	  continue;
-	} else {
-	  if ((refAllele.size() > altAllele.size()) && (_isDNA(refAllele)) && (_isDNA(altAllele))) {
-	    svRec.svt = 2;
-	    tagUse = false;
-	  } else if ((altAllele.size() > refAllele.size()) && (_isDNA(refAllele)) && (_isDNA(altAllele))) {
-	    insertion = true;
-	    svRec.svt = 4;
-	    tagUse = false;
-	  } else continue;
-	}
-	if (tagUse) {
-	  if (bcf_get_info_int32(hdr, rec, "END", &svend, &nsvend) > 0) svRec.svEnd = *svend;
-	  else continue;
-	} else {
-	  if (insertion) {
-	    svRec.svEnd = rec->pos + 2;
-	    int32_t diff =  altAllele.size() - refAllele.size();
-	    svRec.insLen = diff;
-	  } else {
-	    int32_t diff = refAllele.size() - altAllele.size();
-	    svRec.svEnd = rec->pos + diff + 2;
-	    svRec.insLen = 0;
-	  }
-	}
-	std::string chrName = bcf_hdr_id2name(hdr, rec->rid);
-	int32_t tid = bam_name2id(hd, chrName.c_str());
-	svRec.chr = tid;
-	svRec.chr2 = tid;
-	svRec.svStart = rec->pos + 1;
-	svRec.id = svs.size();
-	svRec.alleles = refAllele + "," + altAllele;
-	svRec.precise=true;
-	svRec.peSupport = 0;
-	svRec.homLen = 0;
-	svRec.srSupport = 5;
-	svRec.peMapQuality = 20;
-	svRec.srAlignQuality = 1;
-	svRec.ciposlow = -50;
-	svRec.ciposhigh = 50;
-	svRec.ciendlow = -50;
-	svRec.ciendhigh = 50;
-
-	// Lazy loading of reference sequence
-	if ((seq == NULL) || (tid != lastRefIndex)) {
-	  if (seq != NULL) free(seq);
-	  int32_t seqlen = -1;
-	  seq = faidx_fetch_seq(fai, chrName.c_str(), 0, faidx_seq_len(fai, chrName.c_str()), &seqlen);
-	  lastRefIndex = tid;
-	}
-
-	// Build consensus sequence
-	if ((seq != NULL) && ((svRec.svStart + 15 < svRec.svEnd) || (svRec.insLen >= 15))) {
-	  int32_t buffer = 75;
-	  if (tagUse) {
-	    int32_t prefix = 0;
-	    if (buffer < rec->pos) prefix = rec->pos - buffer;
-	    std::string pref = boost::to_upper_copy(std::string(seq + prefix, seq + rec->pos + 1));
-	    int32_t suffix = svRec.svEnd + buffer;
-	    std::string suf = boost::to_upper_copy(std::string(seq + svRec.svEnd, seq + suffix));
-	    svRec.consensus = pref + suf;
-	  } else {
-	    int32_t prefix = 0;
-	    if (buffer < rec->pos) prefix = rec->pos - buffer;
-	    std::string pref = boost::to_upper_copy(std::string(seq + prefix, seq + rec->pos));
-	    int32_t suffix = svRec.svEnd + buffer;
-	    std::string suf = boost::to_upper_copy(std::string(seq + svRec.svEnd - 1, seq + suffix));
-	    svRec.consensus = pref + altAllele + suf;
-	  }
-	  svs.push_back(svRec);
-	}
-      }
+    } else {
+      std::cerr << "Error: Delly genotyping requires local SV assembly (INFO/CONSENSUS) and breakpoint (INFO/CONSBP) introduced in delly v1.1.7!" << std::endl;
+      break;
     }
   }
   // Clean-up
   free(svend);
   free(pos2);
+  free(consbp);
   free(svlen);
   free(svt);
   free(method);
@@ -385,10 +310,6 @@ vcfParse(TConfig const& c, bam_hdr_t* hd, std::vector<TStructuralVariantRecord>&
   free(ct);
   free(chr2);
 
-  // Clean-up index
-  if (seq != NULL) free(seq);
-  fai_destroy(fai);
-  
   // Close VCF
   bcf_hdr_destroy(hdr);
   bcf_close(ifile);
@@ -437,6 +358,7 @@ vcfOutput(TConfig const& c, std::vector<TStructuralVariantRecord> const& svs, TJ
   bcf_hdr_append(hdr, "##INFO=<ID=SR,Number=1,Type=Integer,Description=\"Split-read support\">");
   bcf_hdr_append(hdr, "##INFO=<ID=SRQ,Number=1,Type=Float,Description=\"Split-read consensus alignment quality\">");
   bcf_hdr_append(hdr, "##INFO=<ID=CONSENSUS,Number=1,Type=String,Description=\"Split-read consensus sequence\">");
+  bcf_hdr_append(hdr, "##INFO=<ID=CONSBP,Number=1,Type=Integer,Description=\"Consensus SV breakpoint position\">");
   bcf_hdr_append(hdr, "##INFO=<ID=CE,Number=1,Type=Float,Description=\"Consensus sequence entropy\">");
   bcf_hdr_append(hdr, "##INFO=<ID=CT,Number=1,Type=String,Description=\"Paired-end signature induced connection type\">");
   bcf_hdr_append(hdr, "##INFO=<ID=SVLEN,Number=1,Type=Integer,Description=\"Insertion length for SVTYPE=INS.\">");
@@ -496,6 +418,14 @@ vcfOutput(TConfig const& c, std::vector<TStructuralVariantRecord> const& svs, TJ
     bcf1_t *rec = bcf_init();
     for(typename TSVs::const_iterator svIter = svs.begin(); svIter!=svs.end(); ++svIter) {
       if ((svIter->srSupport == 0) && (svIter->peSupport == 0)) continue;
+      // In discovery mode, skip SVs that have less than 2 reads support after genotyping
+      if (!c.hasVcfFile) {
+	uint32_t totalGtSup = 0;
+	for(unsigned int file_c = 0; file_c < c.files.size(); ++file_c) {
+	  totalGtSup += spanCountMap[file_c][svIter->id].alt.size() + jctCountMap[file_c][svIter->id].alt.size();
+	}
+	if (totalGtSup < 2) continue;
+      }
       
       // Output main vcf fields
       int32_t tmpi = bcf_hdr_id2int(hdr, BCF_DT_ID, "PASS");
@@ -510,6 +440,7 @@ vcfOutput(TConfig const& c, std::vector<TStructuralVariantRecord> const& svs, TJ
       int32_t svStartPos = svIter->svStart - 1;
       if (svStartPos < 1) svStartPos = 1;
       int32_t svEndPos = svIter->svEnd;
+      if (svIter->chr == svIter->chr2) --svEndPos; // To match VCF spec
       if (svEndPos < 1) svEndPos = 1;
       if (svEndPos >= (int32_t) bamhd->target_len[svIter->chr2]) svEndPos = bamhd->target_len[svIter->chr2] - 1;
       rec->pos = svStartPos;
@@ -572,6 +503,8 @@ vcfOutput(TConfig const& c, std::vector<TStructuralVariantRecord> const& svs, TJ
 	  bcf_update_info_string(hdr, rec, "CONSENSUS", svIter->consensus.c_str());
 	  tmpf = entropy(svIter->consensus);
 	  bcf_update_info_float(hdr, rec, "CE", &tmpf, 1);
+	  tmpi = svIter->consBp;
+	  bcf_update_info_int32(hdr, rec, "CONSBP", &tmpi, 1);
 	}
       }
       
