@@ -2,6 +2,9 @@
 #define SPLIT_H
 
 #include <iostream>
+
+//#include "bindings/cpp/WFAligner.hpp"
+
 #include "edlib.h"
 #include "gotoh.h"
 #include "needle.h"
@@ -21,6 +24,34 @@ namespace torali
     AlignDescriptor() : cStart(0), cEnd(0), rStart(0), rEnd(0), homLeft(0), homRight(0), percId(0) {}
   };
 
+
+  /*
+  template<typename TWFAligner>
+  inline void
+  printWfaAlignment(std::string const& pattern, std::string const& text, TWFAligner& aligner) {
+    std::string cigar = aligner.getAlignmentCigar();
+
+    std::cerr << "Alignment score: " << aligner.getAlignmentScore() << std::endl;
+
+    // pattern
+    uint32_t pidx = 0;
+    for (uint32_t j = 0; j < cigar.size(); ++j) {
+      if (cigar[j] == 'I') std::cerr << '-';
+      else std::cerr << pattern[pidx++];
+    }
+    std::cerr << std::endl;
+
+    // text
+    uint32_t tidx = 0;
+    for (uint32_t j = 0; j < cigar.size(); ++j) {
+      if (cigar[j] == 'D') std::cerr << '-';
+      else std::cerr <<	text[tidx++];
+    }
+    std::cerr << std::endl;
+  }
+  */
+  
+  
   template<typename TBPoint>
   inline void
   _adjustOrientation(std::string& sequence, TBPoint bpPoint, int32_t const svt) {
@@ -175,7 +206,9 @@ namespace torali
       } else if (svt == 3) {
 	return boost::to_upper_copy(std::string(ref + svRec.svEndBeg, ref + svRec.svEndEnd)) + boost::to_upper_copy(std::string(ref + svRec.svStartBeg, ref + svRec.svStartEnd));
       } else if (svt == 0) {
-	std::string strEnd=boost::to_upper_copy(std::string(ref + svRec.svEndBeg, ref + svRec.svEndEnd));
+	std::string strEnd;
+	if ((svRec.svEnd - svRec.svStart) > c.minConsWindow) strEnd = boost::to_upper_copy(std::string(ref + svRec.svEndBeg, ref + svRec.svEndEnd));
+	else strEnd=boost::to_upper_copy(std::string(ref + svRec.svStart, ref + svRec.svEndEnd));
 	std::string strRevComp=strEnd;
 	std::string::reverse_iterator itR = strEnd.rbegin();
 	std::string::reverse_iterator itREnd = strEnd.rend();
@@ -189,9 +222,12 @@ namespace torali
 	  default: break;
 	  }
 	}
-	return boost::to_upper_copy(std::string(ref + svRec.svStartBeg, ref + svRec.svStartEnd)) + strRevComp;
+	if ((svRec.svEnd - svRec.svStart) > c.minConsWindow) return boost::to_upper_copy(std::string(ref + svRec.svStartBeg, ref + svRec.svStartEnd)) + strRevComp;
+	else return boost::to_upper_copy(std::string(ref + svRec.svStartBeg, ref + svRec.svStartEnd)) + strRevComp + boost::to_upper_copy(std::string(ref + svRec.svEnd, ref + svRec.svEndEnd));
       } else if (svt == 1) {
-	std::string strStart=boost::to_upper_copy(std::string(ref + svRec.svStartBeg, ref + svRec.svStartEnd));
+	std::string strStart;
+	if ((svRec.svEnd - svRec.svStart) > c.minConsWindow) strStart = boost::to_upper_copy(std::string(ref + svRec.svStartBeg, ref + svRec.svStartEnd));
+	else strStart = boost::to_upper_copy(std::string(ref + svRec.svStartBeg, ref + svRec.svEnd));
 	std::string strRevComp=strStart;
 	std::string::reverse_iterator itR = strStart.rbegin();
 	std::string::reverse_iterator itREnd = strStart.rend();
@@ -205,7 +241,8 @@ namespace torali
 	  default: break;
 	  }
 	}
-	return strRevComp + boost::to_upper_copy(std::string(ref + svRec.svEndBeg, ref + svRec.svEndEnd));
+	if ((svRec.svEnd - svRec.svStart) > c.minConsWindow) return strRevComp + boost::to_upper_copy(std::string(ref + svRec.svEndBeg, ref + svRec.svEndEnd));
+	else return boost::to_upper_copy(std::string(ref + svRec.svStartBeg, ref + svRec.svStart)) + strRevComp + boost::to_upper_copy(std::string(ref + svRec.svEndBeg, ref + svRec.svEndEnd));
       }
     }
     return "";
@@ -258,16 +295,30 @@ namespace torali
 	finalGapEnd = sv.svEndBeg + ad.rStart;
 	return true;
       } else if (svt == 0) {
-	int32_t annealed = sv.svStartEnd - sv.svStartBeg;
-	if ((ad.rStart >= annealed) || (ad.rEnd < annealed)) return false;
-	finalGapStart = sv.svStartBeg + ad.rStart;
-	finalGapEnd = sv.svEndBeg + (ref.size() - ad.rEnd) + 1;
+	if ((sv.svEnd - sv.svStart) > c.minConsWindow) {
+	  int32_t annealed = sv.svStartEnd - sv.svStartBeg;
+	  if ((ad.rStart >= annealed) || (ad.rEnd < annealed)) return false;
+	  finalGapStart = sv.svStartBeg + ad.rStart;
+	  finalGapEnd = sv.svEndBeg + (ref.size() - ad.rEnd) + 1;
+	} else {
+	  int32_t annealed = sv.svStartEnd - sv.svStartBeg;
+	  if ((ad.rStart >= annealed) || (ad.rEnd < annealed)) return false;
+	  finalGapStart = sv.svStartBeg + ad.rStart;
+	  finalGapEnd = sv.svEndEnd - (ad.rEnd - annealed);
+	}
 	return true;
       } else if (svt == 1) {
-	int32_t annealed = sv.svStartEnd - sv.svStartBeg;
-	if ((ad.rStart >= annealed) || (ad.rEnd < annealed)) return false;
-	finalGapStart = sv.svStartBeg + (annealed - ad.rStart) + 1;
-	finalGapEnd = sv.svEndBeg + (ad.rEnd - annealed);
+	if ((sv.svEnd - sv.svStart) > c.minConsWindow) {
+	  int32_t annealed = sv.svStartEnd - sv.svStartBeg;
+	  if ((ad.rStart >= annealed) || (ad.rEnd < annealed)) return false;
+	  finalGapStart = sv.svStartBeg + (annealed - ad.rStart) + 1;
+	  finalGapEnd = sv.svEndBeg + (ad.rEnd - annealed);
+	} else {
+	  int32_t annealed = (sv.svStart - sv.svStartBeg) + (sv.svEnd - sv.svStartBeg);
+	  if ((ad.rStart >= annealed) || (ad.rEnd < annealed)) return false;
+	  finalGapStart = sv.svStartBeg + (annealed - ad.rStart) + 1;
+	  finalGapEnd = sv.svEndBeg + (ad.rEnd - annealed);
+	}
 	return true;
       } else if (svt == 4) {
 	finalGapStart = sv.svStartBeg + ad.rStart;
@@ -392,9 +443,10 @@ namespace torali
 
     // Is this a valid split-read alignment?
     if (!_validSRAlignment(ad.cStart, ad.cEnd, ad.rStart, ad.rEnd, svt)) return false;
-
+    
     // Check percent identity
     _percentIdentity(align, gS, gE, ad.percId);
+    //std::cerr << ad.percId << ',' << c.flankQuality << std::endl;
     if (ad.percId < c.flankQuality) return false;
 
     // Find homology
@@ -408,14 +460,178 @@ namespace torali
     return true;
   }
 
+
+  inline void
+  editDistanceVec(std::string const& seqI, std::string const& seqJ, EdlibAlignResult& cigar, std::vector<uint32_t>& dist) {
+    // Fill edit distance vector
+    dist.resize(seqI.size());
+    std::fill(dist.begin(), dist.end(), 0);
+
+    // Compute edit distance
+    int32_t tIdx = -1;
+    int32_t qIdx = -1;
+    uint32_t editDist = 0;
+    // seqI
+    for (int32_t j = 0; j < cigar.alignmentLength; ++j) {
+      if (cigar.alignment[j] == EDLIB_EDOP_DELETE) {
+	++tIdx;
+	++editDist;
+      }
+      else if (cigar.alignment[j] == EDLIB_EDOP_INSERT) {
+	++qIdx;
+	++editDist;
+	dist[qIdx] = editDist;
+      }
+      else {
+	++tIdx;
+	++qIdx;
+	if (seqI[qIdx] != seqJ[tIdx]) ++editDist;
+	dist[qIdx] = editDist;
+      }
+    }
+  }
+
+  template<typename TAlign>
+  inline void
+  glueAlignment(std::string const& query, std::string const& target, uint32_t const gaplen, EdlibAlignMode const modeCode, EdlibAlignResult& cigarLeft, EdlibAlignResult& cigarRight, TAlign& align) {
+    // Create new alignment
+    int32_t tIdx = -1;
+    int32_t qIdx = -1;
+    uint32_t missingStart = 0;
+    if (modeCode == EDLIB_MODE_HW) {
+      tIdx = cigarLeft.endLocations[0];
+      for (int32_t i = 0; i < cigarLeft.alignmentLength; i++) {
+	if (cigarLeft.alignment[i] != EDLIB_EDOP_INSERT) --tIdx;
+      }
+      if (tIdx >= 0) missingStart = tIdx + 1;
+    }
+    uint32_t missingEnd = cigarRight.endLocations[0];
+    if (missingEnd < target.size()) missingEnd = target.size() - missingEnd - 1;
+    align.resize(boost::extents[2][missingStart + cigarLeft.alignmentLength + gaplen + cigarRight.alignmentLength + missingEnd]);
+
+    // fix start
+    if (modeCode == EDLIB_MODE_HW) {
+      if (missingStart) {
+	for (uint32_t j = 0; j < missingStart; ++j) {
+	  align[1][j] = target[j];
+	  align[0][j] = '-';
+	}
+      }
+    }
+    // target
+    for (int32_t j = 0; j < cigarLeft.alignmentLength; ++j) {
+      if (cigarLeft.alignment[j] == EDLIB_EDOP_INSERT) align[1][j + missingStart] = '-';
+      else {
+	++tIdx;
+	align[1][j + missingStart] = target[tIdx];
+      }
+    }
+    // query
+    for (int32_t j = 0; j < cigarLeft.alignmentLength; ++j) {
+      if (cigarLeft.alignment[j] == EDLIB_EDOP_DELETE) align[0][j + missingStart] = '-';
+      else align[0][j + missingStart] = query[++qIdx];
+    }
+    // gap
+    for (uint32_t j = 0; j < gaplen; ++j) {
+      align[0][j + missingStart + cigarLeft.alignmentLength] = '-';
+      ++tIdx;
+      align[1][j + missingStart + cigarLeft.alignmentLength] = target[tIdx];
+    }
+    // target
+    for (int32_t j = 0; j < cigarRight.alignmentLength; ++j) {
+      if (cigarRight.alignment[j] == EDLIB_EDOP_INSERT) align[1][j + missingStart + cigarLeft.alignmentLength + gaplen] = '-';
+      else {
+	++tIdx;
+	align[1][j + missingStart + cigarLeft.alignmentLength + gaplen] = target[tIdx];
+      }
+    }
+    // query
+    for (int32_t j = 0; j < cigarRight.alignmentLength; ++j) {
+      if (cigarRight.alignment[j] == EDLIB_EDOP_DELETE) align[0][j + missingStart + cigarLeft.alignmentLength + gaplen] = '-';
+      else align[0][j + missingStart + cigarLeft.alignmentLength + gaplen] = query[++qIdx];
+    }
+    // fix end
+    if (modeCode == EDLIB_MODE_HW) {
+      if (missingEnd) {
+	for (uint32_t j = 0; j < missingEnd; ++j) {
+	  ++tIdx;
+	  align[1][j + missingStart + cigarLeft.alignmentLength + gaplen + cigarRight.alignmentLength] = target[tIdx];
+	  align[0][j + missingStart + cigarLeft.alignmentLength + gaplen + cigarRight.alignmentLength] = '-';
+	}
+      }
+    }
+  }
+
+
+  template<typename TAlign>
+  inline bool
+  splitAlign(std::string const& cons, std::string const& svRefStr, TAlign& align) {
+    // Prepare SHW alignment
+    std::string prefix = svRefStr.substr(0, svRefStr.size()/3);
+    EdlibAlignResult cigarPrefix = edlibAlign(prefix.c_str(), prefix.size(), cons.c_str(), cons.size(), edlibNewAlignConfig(-1, EDLIB_MODE_HW, EDLIB_TASK_PATH, NULL, 0));
+    uint32_t csStart = infixStart(cigarPrefix);
+    edlibFreeAlignResult(cigarPrefix);
+    std::string suffix = svRefStr.substr(2 * svRefStr.size()/3);
+    EdlibAlignResult cigarSuffix = edlibAlign(suffix.c_str(), suffix.size(), cons.c_str(), cons.size(), edlibNewAlignConfig(-1, EDLIB_MODE_HW, EDLIB_TASK_PATH, NULL, 0));
+    uint32_t csEnd = infixEnd(cigarSuffix);
+    edlibFreeAlignResult(cigarSuffix);
+    
+    if (csStart >= csEnd) return false;
+    std::string cs = cons.substr(csStart, (csEnd - csStart));
+    
+    // Prefix alignment
+    EdlibAlignResult cigar = edlibAlign(svRefStr.c_str(), svRefStr.size(), cs.c_str(), cs.size(), edlibNewAlignConfig(-1, EDLIB_MODE_SHW, EDLIB_TASK_PATH, NULL, 0));
+    std::vector<uint32_t> distFwd;
+    //printAlignment(svRefStr, cs, EDLIB_MODE_HW, cigar);
+    editDistanceVec(svRefStr, cs, cigar, distFwd);
+    edlibFreeAlignResult(cigar);
+    // Suffix alignment
+    std::string svRefStrRev = svRefStr;
+    std::string csRev = cs;
+    reverseComplement(svRefStrRev);
+    reverseComplement(csRev);
+    EdlibAlignResult cigarRev = edlibAlign(svRefStrRev.c_str(), svRefStrRev.size(), csRev.c_str(), csRev.size(), edlibNewAlignConfig(-1, EDLIB_MODE_SHW, EDLIB_TASK_PATH, NULL, 0));
+    std::vector<uint32_t> distRev;
+    //printAlignment(svRefStrRev, csRev, EDLIB_MODE_HW, cigarRev);
+    editDistanceVec(svRefStrRev, csRev, cigarRev, distRev);
+    edlibFreeAlignResult(cigarRev);
+    
+    // Best join
+    uint32_t bestJoin = 0;
+    for(uint32_t i = 1; i < distFwd.size() - 1; ++i) {
+      if (distFwd[i] + distRev[distFwd.size() - i - 2] < distFwd[bestJoin] + distRev[distFwd.size() - bestJoin - 2]) bestJoin = i;
+    }
+    
+    // Split reference and do infix alignment
+    std::string svRefLeft = svRefStr.substr(0, bestJoin + 1);
+    std::string svRefRight = svRefStr.substr(bestJoin + 1);
+    
+    // Infix alignment
+    EdlibAlignResult cigarLeft = edlibAlign(svRefLeft.c_str(), svRefLeft.size(), cons.c_str(), cons.size(), edlibNewAlignConfig(-1, EDLIB_MODE_HW, EDLIB_TASK_PATH, NULL, 0));
+    uint32_t leftEnd = infixEnd(cigarLeft);
+    //printAlignment(svRefLeft, cons, EDLIB_MODE_HW, cigarLeft);
+    EdlibAlignResult cigarRight = edlibAlign(svRefRight.c_str(), svRefRight.size(), cons.c_str(), cons.size(), edlibNewAlignConfig(-1, EDLIB_MODE_HW, EDLIB_TASK_PATH, NULL, 0));
+    //printAlignment(svRefRight, cons, EDLIB_MODE_HW, cigarRight);
+    uint32_t rightStart = infixStart(cigarRight);
+    
+    // Glue alignment
+    if (leftEnd + 15 >= rightStart) return false;
+    glueAlignment(svRefStr, cons, rightStart - leftEnd - 1, EDLIB_MODE_HW, cigarLeft, cigarRight, align);
+    edlibFreeAlignResult(cigarLeft);
+    edlibFreeAlignResult(cigarRight);
+    return true;
+  }
+  
+
   template<typename TAlign>
   inline bool
   _consRefAlignment(std::string const& cons, std::string const& svRefStr, TAlign& aln, int32_t const svt) {
     AlignConfig<true, false> semiglobal;
-    DnaScore<int> lnsc(5, -4, -4, -4);
+    DnaScore<int> lnsc(1, -1, -1, -1);
     bool reNeedle = false;
     if (svt == 4) {
-      reNeedle = longNeedle(svRefStr, cons, aln, semiglobal, lnsc);
+      reNeedle = splitAlign(cons, svRefStr, aln);
+      //reNeedle = longNeedle(svRefStr, cons, aln, semiglobal, lnsc);
       for(uint32_t j = 0; j < aln.shape()[1]; ++j) {
 	char tmp = aln[0][j];
 	aln[0][j] = aln[1][j];
@@ -427,6 +643,90 @@ namespace torali
     return reNeedle;
   }
 
+  template<typename TConfig>
+  inline bool
+  _alignConsensus(TConfig const& c, std::string& consensus, std::string& svRefStr, StructuralVariantRecord& sv, Breakpoint& bp, bool const realign) {
+    // Realign?
+    if (realign) {
+      std::string revc = consensus;
+      reverseComplement(revc);
+      EdlibAlignResult alignFwd = edlibAlign(svRefStr.c_str(), svRefStr.size(), consensus.c_str(), consensus.size(), edlibNewAlignConfig(-1, EDLIB_MODE_NW, EDLIB_TASK_DISTANCE, NULL, 0));
+      EdlibAlignResult alignRev = edlibAlign(svRefStr.c_str(), svRefStr.size(), revc.c_str(), revc.size(), edlibNewAlignConfig(-1, EDLIB_MODE_NW, EDLIB_TASK_DISTANCE, NULL, 0));
+      if (alignRev.editDistance < alignFwd.editDistance) consensus = revc;
+      edlibFreeAlignResult(alignFwd);
+      edlibFreeAlignResult(alignRev);
+    }
+
+    //wfa::WFAlignerGapAffine2Pieces aligner(8, 12, 4, 60, 1, wfa::WFAligner::Alignment, wfa::WFAligner::MemoryHigh);
+    //aligner.alignEnd2End(consensus, svRefStr);
+    //printWfaAlignment(consensus, svRefStr, aligner);
+
+    // Consensus to reference alignment
+    typedef boost::multi_array<char, 2> TAlign;
+    TAlign align;
+    //std::cerr << "Consensus-to-Reference alignment" << std::endl;
+    if (!_consRefAlignment(consensus, svRefStr, align, sv.svt)) return false;
+    
+    // Debug consensus to reference alignment
+    //for(uint32_t i = 0; i < align.shape()[0]; ++i) {
+    //for(uint32_t j = 0; j< align.shape()[1]; ++j) {
+    //std::cerr << align[i][j];
+    //}
+    //std::cerr << std::endl;
+    //}
+    //std::cerr << std::endl;
+    //std::cerr << std::endl;
+
+    // Check breakpoint
+    AlignDescriptor ad;
+    if (!_findSplit(c, consensus, svRefStr, align, ad, sv.svt)) return false;
+
+    // Transform coordinates
+    unsigned int finalGapStart = 0;
+    unsigned int finalGapEnd = 0;
+    if (!_coordTransform(c, svRefStr, bp, ad, finalGapStart, finalGapEnd, sv.svt)) return false;
+    
+    if ((_translocation(sv.svt)) || (finalGapStart < finalGapEnd)) {
+
+      // Get exact alleles for INS and DEL
+      if (sv.svEnd - sv.svStart <= c.indelsize) {
+	if ((sv.svt == 2) || (sv.svt == 4)) {
+	  std::string refVCF;
+	  std::string altVCF;
+	  int32_t cpos = 0;
+	  bool inSV = false;
+	  for(uint32_t j = 0; j<align.shape()[1]; ++j) {
+	    if (align[0][j] != '-') {
+	      ++cpos;
+	      if (cpos == ad.cStart) inSV = true;
+	      else if (cpos == ad.cEnd) inSV = false;
+	    }
+	    if (inSV) {
+	      if (align[0][j] != '-') altVCF += align[0][j];
+	      if (align[1][j] != '-') refVCF += align[1][j];
+	    }
+	  }
+	  sv.alleles = _addAlleles(refVCF, altVCF);
+	}
+      }
+      sv.precise=true;
+      sv.svStart=finalGapStart;
+      sv.svEnd=finalGapEnd;
+      sv.srAlignQuality = ad.percId;
+      sv.insLen=ad.cEnd - ad.cStart - 1;
+      sv.consBp = ad.cStart;
+      sv.homLen=std::max(0, ad.homLeft + ad.homRight - 2);
+      int32_t ci_wiggle = std::max(ad.homLeft, ad.homRight);
+      sv.ciposlow = -ci_wiggle;
+      sv.ciposhigh = ci_wiggle;
+      sv.ciendlow = -ci_wiggle;
+      sv.ciendhigh = ci_wiggle;
+      return true;
+    } else {
+      return false;
+    }
+  }
+  
   template<typename TConfig>
   inline bool
   alignConsensus(TConfig const& c, bam_hdr_t* hdr, char const* seq, char const* sndSeq, StructuralVariantRecord& sv, bool const realign) {
@@ -440,55 +740,15 @@ namespace torali
     } else _initBreakpoint(hdr, bp, sv.consensus.size(), sv.svt);
     if (bp.chr != bp.chr2) bp.part1 = _getSVRef(c, sndSeq, bp, bp.chr2, sv.svt);
     std::string svRefStr = _getSVRef(c, seq, bp, bp.chr, sv.svt);
-    
-    // Realign?
-    if (realign) {
-      std::string revc = sv.consensus;
-      reverseComplement(revc);
-      EdlibAlignResult alignFwd = edlibAlign(sv.consensus.c_str(), sv.consensus.size(), svRefStr.c_str(), svRefStr.size(), edlibNewAlignConfig(-1, EDLIB_MODE_NW, EDLIB_TASK_DISTANCE, NULL, 0));
-      EdlibAlignResult alignRev = edlibAlign(revc.c_str(), revc.size(), svRefStr.c_str(), svRefStr.size(), edlibNewAlignConfig(-1, EDLIB_MODE_NW, EDLIB_TASK_DISTANCE, NULL, 0));
-      if (alignRev.editDistance < alignFwd.editDistance) sv.consensus = revc;
-      edlibFreeAlignResult(alignFwd);
-      edlibFreeAlignResult(alignRev);
-    }
-    
-    
-    // Consensus to reference alignment
-    typedef boost::multi_array<char, 2> TAlign;
-    TAlign align;
-    if (!_consRefAlignment(sv.consensus, svRefStr, align, sv.svt)) return false;
 
-    // Debug consensus to reference alignment
-    //std::cerr << "Consensus-to-Reference alignment" << std::endl;
-    //for(uint32_t i = 0; i < align.shape()[0]; ++i) {
-    //for(uint32_t j = 0; j< align.shape()[1]; ++j) {
-    //std::cerr << align[i][j];
-    //}
-    //std::cerr << std::endl;
-    //}
-    
-    // Check breakpoint
-    AlignDescriptor ad;
-    if (!_findSplit(c, sv.consensus, svRefStr, align, ad, sv.svt)) return false;
-    
-    // Get the start and end of the structural variant
-    unsigned int finalGapStart = 0;
-    unsigned int finalGapEnd = 0;
-    if (!_coordTransform(c, svRefStr, bp, ad, finalGapStart, finalGapEnd, sv.svt)) return false;
-    
-    sv.precise=true;
-    sv.svStart=finalGapStart;
-    sv.svEnd=finalGapEnd;
-    sv.srAlignQuality = ad.percId;
-    sv.insLen=ad.cEnd - ad.cStart - 1;
-    sv.homLen=std::max(0, ad.homLeft + ad.homRight - 2);
-    int32_t ci_wiggle = std::max(ad.homLeft, ad.homRight);
-    sv.ciposlow = -ci_wiggle;
-    sv.ciposhigh = ci_wiggle;
-    sv.ciendlow = -ci_wiggle;
-    sv.ciendhigh = ci_wiggle;
+    // Debug
+    //std::cerr << ">Cons:" << sv.svStart << std::endl;
+    //std::cerr << sv.consensus << std::endl;
+    //std::cerr << ">Ref:" << sv.svStart << std::endl;
+    //std::cerr << svRefStr << std::endl;
 
-    return true;
+    // Generate consensus alignment
+    return _alignConsensus(c, sv.consensus, svRefStr, sv, bp, realign);
   }
 
   template<typename TConfig>
