@@ -46,10 +46,11 @@ namespace torali {
     uint32_t graphPruning;
     uint32_t minCliqueSize;
     uint32_t maxReadPerSV;
+    uint32_t maxGenoReadCount;
+    int32_t minConsWindow;
     int32_t nchr;
     int32_t minimumFlankSize;
     int32_t indelsize;
-    int32_t maxInsertionSize;
     float indelExtension;
     float flankQuality;
     std::set<int32_t> svtset;
@@ -63,23 +64,6 @@ namespace torali {
     std::vector<std::string> sampleName;
   };
   
-  template<typename TConfig>
-  inline void
-  _alignmentScore(TConfig& c, std::string const& scoring) {
-    typedef boost::tokenizer< boost::char_separator<char> > Tokenizer;
-    boost::char_separator<char> sep(",\t ");
-    Tokenizer tokens(scoring, sep);
-    Tokenizer::iterator tokIter = tokens.begin();
-    int32_t match = boost::lexical_cast<int32_t>(*tokIter++);
-    int32_t mismatch = boost::lexical_cast<int32_t>(*tokIter++);
-    int32_t go = boost::lexical_cast<int32_t>(*tokIter++);
-    int32_t ge = boost::lexical_cast<int32_t>(*tokIter++);
-    c.aliscore = DnaScore<int>(match, mismatch, go, ge);
-    //std::cerr << c.aliscore.match << ',' << c.aliscore.mismatch << ',' << c.aliscore.go << ',' << c.aliscore.ge << std::endl;
-    return;
-  }
-
-
 
  template<typename TConfig>
  inline int32_t
@@ -131,13 +115,19 @@ namespace torali {
      // Sort SVs
      sort(svc.begin(), svc.end(), SortSVs<StructuralVariantRecord>());
       
-     // Keep assembled SVs only
+     // Remove duplicates
      StructuralVariantRecord lastSV;
      for(typename TVariants::iterator svIter = svc.begin(); svIter != svc.end(); ++svIter) {
        if ((svIter->srSupport == 0) && (svIter->peSupport == 0)) continue;
        // Duplicate?
        if (!svs.empty()) {
-	 if ((lastSV.chr == svIter->chr) && (lastSV.chr2 == svIter->chr2) && (lastSV.svt == svIter->svt) && (std::abs(svIter->svStart - lastSV.svStart) < c.minRefSep) && (std::abs(svIter->svEnd - lastSV.svEnd) < c.minRefSep)) continue;
+	 if ((lastSV.chr == svIter->chr) && (lastSV.chr2 == svIter->chr2) && (lastSV.svt == svIter->svt) && (std::abs(svIter->svStart - lastSV.svStart) < c.minRefSep) && (std::abs(svIter->svEnd - lastSV.svEnd) < c.minRefSep)) {
+	   // Check length
+	   int32_t lengthvar = std::min(0.1 * (svIter->svEnd - svIter->svStart), 0.1 * (lastSV.svEnd - lastSV.svStart));
+	   int32_t lengthdiff = std::min(svIter->svEnd - svIter->svStart, lastSV.svEnd - lastSV.svStart);
+	   if (lengthvar < 15) lengthvar = 15;
+	   if (lengthdiff < lengthvar) continue;
+	 }
        }
        lastSV = *svIter;
        svs.push_back(*svIter);
@@ -149,7 +139,6 @@ namespace torali {
      // Re-number SVs
      uint32_t cliqueCount = 0;
      for(typename TVariants::iterator svIt = svs.begin(); svIt != svs.end(); ++svIt, ++cliqueCount) svIt->id = cliqueCount;
-     //outputStructuralVariants(c, svs);
    } else vcfParse(c, hdr, svs);
    // Clean-up
    bam_hdr_destroy(hdr);
@@ -176,10 +165,10 @@ namespace torali {
      spanMap[file_c].resize(svs.size(), SpanningCount());
      rcMap[file_c].resize(svs.size(), ReadCount());
    }
-      
+
    // SV Genotyping
    genotypeLR(c, svs, jctMap, rcMap);
-
+   
    // VCF Output
    vcfOutput(c, svs, jctMap, rcMap, spanMap);
 
@@ -213,7 +202,7 @@ namespace torali {
    
    boost::program_options::options_description disc("Discovery options");
    disc.add_options()
-     ("mapqual,q", boost::program_options::value<uint16_t>(&c.minMapQual)->default_value(10), "min. mapping quality")
+     ("mapqual,q", boost::program_options::value<uint16_t>(&c.minMapQual)->default_value(1), "min. mapping quality")
      ("minclip,c", boost::program_options::value<uint32_t>(&c.minClip)->default_value(25), "min. clipping length")
      ("min-clique-size,z", boost::program_options::value<uint32_t>(&c.minCliqueSize)->default_value(3), "min. clique size")     
      ("minrefsep,m", boost::program_options::value<uint32_t>(&c.minRefSep)->default_value(30), "min. reference separation")
@@ -223,16 +212,17 @@ namespace torali {
    boost::program_options::options_description cons("Consensus options");
    cons.add_options()
      ("max-reads,p", boost::program_options::value<uint32_t>(&c.maxReadPerSV)->default_value(15), "max. reads for consensus computation")
+     ("cons-window,w", boost::program_options::value<int32_t>(&c.minConsWindow)->default_value(1000), "consensus window")
      ("flank-size,f", boost::program_options::value<int32_t>(&c.minimumFlankSize)->default_value(100), "min. flank size")
      ("flank-quality,a", boost::program_options::value<float>(&c.flankQuality)->default_value(0.9), "min. flank quality")
      ("indel-size,i", boost::program_options::value<int32_t>(&c.indelsize)->default_value(10000), "use exact alleles for InDels <10kbp")
-     ("max-isize,r", boost::program_options::value<int32_t>(&c.maxInsertionSize)->default_value(10000), "max. insertion size")
-     ;     
+     ;
    
    boost::program_options::options_description geno("Genotyping options");
    geno.add_options()
      ("vcffile,v", boost::program_options::value<boost::filesystem::path>(&c.vcffile), "input VCF/BCF file for genotyping")
      ("geno-qual,u", boost::program_options::value<uint16_t>(&c.minGenoQual)->default_value(5), "min. mapping quality for genotyping")
+     ("max-geno-count,b", boost::program_options::value<uint32_t>(&c.maxGenoReadCount)->default_value(250), "max. reads aligned for SR genotyping")
      ("dump,d", boost::program_options::value<boost::filesystem::path>(&c.dumpfile), "gzipped output file for SV-reads")
      ;
 
