@@ -10,6 +10,8 @@
 #include <htslib/sam.h>
 #include <sstream>
 #include <math.h>
+
+#include "edlib.h"
 #include "tags.h"
 
 
@@ -77,6 +79,116 @@ namespace torali
   };
 
 
+  inline uint32_t
+  infixStart(EdlibAlignResult& cigar) {
+    int32_t tIdx = cigar.endLocations[0];
+    for (int32_t i = 0; i < cigar.alignmentLength; i++) {
+      if (cigar.alignment[i] != EDLIB_EDOP_INSERT) tIdx--;
+    }
+    if (tIdx >= 0) return tIdx + 1;
+    else return 0;
+  }
+
+  inline uint32_t
+  infixEnd(EdlibAlignResult& cigar) {
+    return cigar.endLocations[0];
+  }
+  
+  inline void
+  printAlignmentPretty(std::string const& query, std::string const& target, EdlibAlignMode const modeCode, EdlibAlignResult& align) {
+    int32_t tIdx = -1;
+    int32_t qIdx = -1;
+    if (modeCode == EDLIB_MODE_HW) {
+        tIdx = align.endLocations[0];
+        for (int32_t i = 0; i < align.alignmentLength; i++) {
+            if (align.alignment[i] != EDLIB_EDOP_INSERT) tIdx--;
+        }
+    }
+    std::cerr << std::endl;
+    for (int start = 0; start < align.alignmentLength; start += 50) {
+      std::cerr << "T: ";
+      int32_t startTIdx = -1;
+      for (int32_t j = start; ((j < start + 50) && (j < align.alignmentLength)); ++j) {
+	if (align.alignment[j] == EDLIB_EDOP_INSERT) std::cerr << "-";
+	else std::cerr << target[++tIdx];
+	if (j == start) startTIdx = tIdx;
+      }
+      std::cerr << " (" << std::max(startTIdx, 0) << " - " << tIdx << ")" << std::endl;
+
+      // match / mismatch
+      std::cerr << ("   ");
+      for (int32_t j = start; j < start + 50 && j < align.alignmentLength; j++) {
+	if (align.alignment[j] == EDLIB_EDOP_MATCH) std::cerr <<  "|";
+	else std::cerr << " ";
+      }
+      std::cerr << std::endl;
+
+      // query
+      std::cerr << "Q: ";
+      int32_t startQIdx = qIdx;
+      for (int32_t j = start; j < start + 50 && j < align.alignmentLength; j++) {
+	if (align.alignment[j] == EDLIB_EDOP_DELETE) std::cerr << "-";
+	else std::cerr << query[++qIdx];
+	if (j == start) startQIdx = qIdx;
+      }
+      std::cerr << " ("<< std::max(startQIdx, 0) << " - " << qIdx << ")" << std::endl;
+      std::cerr << std::endl;
+    }
+  }
+
+  inline void
+  printAlignment(std::string const& seqI, std::string const& seqJ, EdlibAlignMode const modeCode, EdlibAlignResult& cigar) {
+    int32_t tIdx = -1;
+    int32_t qIdx = -1;
+    uint32_t missingEnd = 0;
+    uint32_t missingStart = 0;
+    if ((modeCode == EDLIB_MODE_HW) || (modeCode == EDLIB_MODE_SHW)) {
+      tIdx = cigar.endLocations[0];
+      if (tIdx < (int32_t) seqJ.size()) missingEnd = seqJ.size() - tIdx - 1;
+      for (int32_t i = 0; i < cigar.alignmentLength; i++) {
+	if (cigar.alignment[i] != EDLIB_EDOP_INSERT) tIdx--;
+      }
+      if (tIdx >= 0) missingStart = tIdx + 1;
+    }
+    // infix alignment, fix start
+    if ((modeCode == EDLIB_MODE_HW) || (modeCode == EDLIB_MODE_SHW)) {
+      if (missingStart) {
+	for (uint32_t j = 0; j < missingStart; ++j) std::cerr << '-';
+      }
+    }
+    // seqI
+    for (int32_t j = 0; j < cigar.alignmentLength; ++j) {
+      if (cigar.alignment[j] == EDLIB_EDOP_DELETE) std::cerr << '-';
+      else std::cerr << seqI[++qIdx];
+    }
+    // infix alignment, fix end
+    if ((modeCode == EDLIB_MODE_HW) || (modeCode == EDLIB_MODE_SHW)) {
+      if (missingEnd) {
+	for (uint32_t j = 0; j < missingEnd; ++j) std::cerr << '-';
+      }
+    }
+    std::cerr << std::endl;
+    // infix alignment, fix start
+    if ((modeCode == EDLIB_MODE_HW) || (modeCode == EDLIB_MODE_SHW)) {
+      if (missingStart) {
+	for (uint32_t j = 0; j < missingStart; ++j) std::cerr << seqJ[j];
+      }
+    }
+    // seqJ
+    for (int32_t j = 0; j < cigar.alignmentLength; ++j) {
+      if (cigar.alignment[j] == EDLIB_EDOP_INSERT) std::cerr << '-';
+      else std::cerr << seqJ[++tIdx];
+    }
+    // infix alignment, fix end
+    if ((modeCode == EDLIB_MODE_HW) || (modeCode == EDLIB_MODE_SHW)) {
+      if (missingEnd) {
+	for (uint32_t j = 0; j < missingEnd; ++j) std::cerr << seqJ[++tIdx];
+      }
+    }
+    std::cerr << std::endl;
+  }
+
+  
   template<typename TConfig>
   inline void
   checkSampleNames(TConfig& c) {
@@ -178,6 +290,19 @@ namespace torali
     else return "NtoN";
   }
 
+  inline bool
+  is_gz(boost::filesystem::path const& f) {
+    std::ifstream bfile(f.string().c_str(), std::ios_base::binary | std::ios::ate);
+    bfile.seekg(0, std::ios::beg);
+    char byte1;
+    bfile.read(&byte1, 1);
+    char byte2;
+    bfile.read(&byte2, 1);
+    bfile.close();
+    if ((byte1 == '\x1F') && (byte2 == '\x8B')) return true;
+    else return false;
+  }
+
   // Output directory/file checks
   inline bool
   _outfileValid(boost::filesystem::path const& outfile) {
@@ -272,6 +397,44 @@ namespace torali
     return alen;
   }
 
+  inline double
+  _sharedCarriers(std::vector<int32_t> const& gt1, std::vector<int32_t> const& gt2) {
+    // Percentage of shared carriers
+    uint32_t carnum = 0;
+    uint32_t carshared = 0;
+    for(uint32_t k = 0; k < gt1.size(); ++k) {
+      if ((gt1[k] != 0) || (gt2[k] != 0)) {
+	++carnum;
+	if ((gt1[k] != 0) && (gt2[k] != 0)) ++carshared;
+      }
+    }
+    return (double) (carshared) / (double) (carnum);
+  }
+
+  inline double
+  nonrefGtConc(std::vector<int32_t> const& gt1, std::vector<int32_t> const& gt2) {
+    uint32_t totgt = 0;
+    uint32_t matchgt = 0;
+    for(uint32_t k = 0; k < gt1.size(); ++k) {
+      if ((gt1[k] != 0) || (gt2[k] != 0)) {
+	++totgt;
+	if (gt1[k] == gt2[k]) ++matchgt;
+      }
+    }
+    return (double) (matchgt) / (double) (totgt);
+  }
+  
+  inline double
+  gtConc(std::vector<int32_t> const& gt1, std::vector<int32_t> const& gt2) {
+    uint32_t totgt = 0;
+    uint32_t matchgt = 0;
+    for(uint32_t k = 0; k < gt1.size(); ++k) {
+      ++totgt;
+      if (gt1[k] == gt2[k]) ++matchgt;
+    }
+    return (double) (matchgt) / (double) (totgt);
+  }
+  
   inline uint32_t halfAlignmentLength(bam1_t const* rec) {
     return (alignmentLength(rec) / 2);
   }
@@ -299,6 +462,15 @@ namespace torali
     return seed;
   }
 
+  inline std::size_t hash_sr(bam1_t* rec) {
+    boost::hash<std::string> string_hash;
+    std::string qname = bam_get_qname(rec);
+    std::size_t seed = hash_string(qname.c_str());
+    boost::hash_combine(seed, string_hash(qname));
+    if ((rec->core.flag & BAM_FREAD1) && (seed > 0))  --seed;
+    return seed;
+  }
+
   inline std::size_t hash_lr(bam1_t* rec) {
     boost::hash<std::string> string_hash;
     std::string qname = bam_get_qname(rec);
@@ -306,6 +478,13 @@ namespace torali
     boost::hash_combine(seed, string_hash(qname));
     return seed;
   }
+
+  inline std::size_t hash_lr(std::string const& qname) {
+    boost::hash<std::string> string_hash;
+    std::size_t seed = hash_string(qname.c_str());
+    boost::hash_combine(seed, string_hash(qname));
+    return seed;
+  }
   
   inline void
   reverseComplement(std::string& sequence) {
